﻿//
//  コンピュータアニメーション特論
//  幾何形状データ（Obj形式）の読み込み＆描画のサンプルプログラム
//


// 基本的なヘッダファイルのインクルード
#ifdef _WIN32
	#include <windows.h>
#endif
#include <stdio.h>
#include <math.h>

// GLUTヘッダファイルのインクルード
#include <GL/glut.h>

// 幾何形状データ（Obj形式）のデータ構造と関数定義のヘッダファイルをインクルード
#include "obj.h"


// 視点操作のための変数
float  camera_yaw = -30.0;    // Ｙ軸を中心とする回転角度
float  camera_pitch = -30.0;  // Ｘ軸を中心とする回転角度
float  camera_distance = 15.0;// 中心からカメラの距離

// マウスのドラッグのための変数
int  drag_mouse_r = 0;  // 右ボタンをドラッグ中かどうかのフラグ（0:非ドラッグ中,1:ドラッグ中）
int  drag_mouse_l = 0;  // 左ボタンをドラッグ中かどうかのフラグ（0:非ドラッグ中,1:ドラッグ中）
int  last_mouse_x;      // 最後に記録されたマウスカーソルのＸ座標
int  last_mouse_y;      // 最後に記録されたマウスカーソルのＹ座標

// 幾何形状オブジェクト
struct  Obj *  obj = NULL;
float  object_y = 0.0f;



//
//  格子模様の床を描画
//
void  DrawFloor( int tile_size, int num_x, int num_z, float r0, float g0, float b0, float r1, float g1, float b1 )
{
	int  x, z;
	float  ox, oz;

	glBegin( GL_QUADS );
	glNormal3d( 0.0, 1.0, 0.0 );

	ox = - ( num_x * tile_size ) / 2;
	for ( x=0; x<num_x; x++ )
	{
		oz = - ( num_z * tile_size ) / 2;
		for ( z=0; z<num_z; z++ )
		{
			if ( ( (x + z) % 2 ) == 0 )
				glColor3f( r0, g0, b0 );
			else
				glColor3f( r1, g1, b1 );

			glTexCoord2d( 0.0f, 0.0f );
			glVertex3d( ox, 0.0, oz );
			glTexCoord2d( 0.0f, 1.0f );
			glVertex3d( ox, 0.0, oz + tile_size );
			glTexCoord2d( 1.0f, 1.0f );
			glVertex3d( ox + tile_size, 0.0, oz + tile_size );
			glTexCoord2d( 1.0f, 0.0f );
			glVertex3d( ox + tile_size, 0.0, oz );

			oz += tile_size;
		}
		ox += tile_size;
	}
	glEnd();
}


//
//  ウィンドウ再描画時に呼ばれるコールバック関数
//
void  DisplayCallback( void )
{
	// 画面をクリア（ピクセルデータとＺバッファの両方をクリア）
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	// 変換行列を設定（ワールド座標系→カメラ座標系）
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	glTranslatef( 0.0,  0.0, - camera_distance );
	glRotatef( - camera_pitch, 1.0, 0.0, 0.0 );
	glRotatef( - camera_yaw, 0.0, 1.0, 0.0 );

	// 光源位置を設定（モデルビュー行列の変更にあわせて再設定）
	float  light0_position[] = { 10.0, 30.0, 10.0, 1.0 };
	glLightfv( GL_LIGHT0, GL_POSITION, light0_position );

	// 格子模様の床を描画
	DrawFloor( 1.0f, 10, 10, 1.0, 1.0, 1.0, 1.0, 0.8, 0.8 );

	// 幾何形状を描画
	if ( obj )
	{
		glColor3f( 1.0, 1.0, 1.0 );

		// 変換行列を設定（物体のモデル座標系→カメラ座標系）
		//（幾何形状が地面に接するように、幾何形状の中心の y座標を設定する）
		glTranslatef( 0.0f, - object_y, 0.0f );

		// 幾何形状を描画
		RenderObj( obj );
	}

	// バックバッファに描画した画面をフロントバッファに表示
	glutSwapBuffers();
}


//
//  ウィンドウサイズ変更時に呼ばれるコールバック関数
//
void  ReshapeCallback( int w, int h )
{
	// ウィンドウ内の描画を行う範囲を設定（ウィンドウ全体に描画するように設定）
	glViewport(0, 0, w, h);

	// カメラ座標系→スクリーン座標系への変換行列を設定
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45, (double)w/h, 1, 500 );
}


//
//  マウスクリック時に呼ばれるコールバック関数
//
void  MouseClickCallback( int button, int state, int mx, int my )
{
	// 右ボタンが押されたらドラッグ開始のフラグを設定
	if ( ( button == GLUT_LEFT_BUTTON ) && ( state == GLUT_DOWN ) )
		drag_mouse_l = 1;
	// 右ボタンが離されたらドラッグ終了のフラグを設定
	else if ( ( button == GLUT_LEFT_BUTTON ) && ( state == GLUT_UP ) )
		drag_mouse_l = 0;

	// 右ボタンが押されたらドラッグ開始のフラグを設定
	if ( ( button == GLUT_RIGHT_BUTTON ) && ( state == GLUT_DOWN ) )
		drag_mouse_r = 1;
	// 右ボタンが離されたらドラッグ終了のフラグを設定
	else if ( ( button == GLUT_RIGHT_BUTTON ) && ( state == GLUT_UP ) )
		drag_mouse_r = 0;

	// 現在のマウス座標を記録
	last_mouse_x = mx;
	last_mouse_y = my;
}


//
//  マウスドラッグ時に呼ばれるコールバック関数
//
void  MouseDragCallback( int mx, int my )
{
	// 左ボタンのドラッグ中であれば、マウスの移動量に応じて視点を移動する
	if ( drag_mouse_l == 1 )
	{
		// マウスの縦移動に応じて距離を移動
		camera_distance += ( my - last_mouse_y ) * 0.2;
		if ( camera_distance < 5.0 )
			camera_distance = 5.0;
	}
	
	// 右ボタンのドラッグ中であれば、マウスの移動量に応じて視点を回転する
	if ( drag_mouse_r == 1 )
	{
		// マウスの横移動に応じてＹ軸を中心に回転
		camera_yaw -= ( mx - last_mouse_x ) * 1.0;
		if ( camera_yaw < 0.0 )
			camera_yaw += 360.0;
		else if ( camera_yaw > 360.0 )
			camera_yaw -= 360.0;
		
		// マウスの縦移動に応じてＸ軸を中心に回転
		camera_pitch -= ( my - last_mouse_y ) * 1.0;
		if ( camera_pitch < -90.0 )
			camera_pitch = -90.0;
		else if ( camera_pitch > 0.0 )
			camera_pitch = 0.0;
	}

	// 今回のマウス座標を記録
	last_mouse_x = mx;
	last_mouse_y = my;

	// 再描画の指示を出す（この後で再描画のコールバック関数が呼ばれる）
	glutPostRedisplay();
}


//
//  キーボードのキーが押されたときに呼ばれるコールバック関数
//
void  KeyboardCallback( unsigned char key, int mx, int my )
{
	// l キーで幾何形状を読み込み
	if ( key == 'l' )
	{
		const int  file_name_len = 256;
		char  file_name[ file_name_len ] = "";

		// ファイルダイアログの設定
		OPENFILENAME	open_file;
		memset( &open_file, 0, sizeof(OPENFILENAME) );
		open_file.lStructSize = sizeof(OPENFILENAME);
		open_file.hwndOwner = NULL;
		open_file.lpstrFilter = "Obj File (*.obj)\0*.obj\0すべて(*.*)\0*.*\0";
		open_file.nFilterIndex = 1;
		open_file.lpstrFile = file_name;
		open_file.nMaxFile = file_name_len;
		open_file.lpstrTitle = "Obj File";
		open_file.lpstrDefExt = "obj";
		open_file.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;

		// ファイルダイアログを表示
		BOOL  ret = GetOpenFileName( &open_file );

		// ファイルが指定されたら新しい幾何形状を読み込み
		if( ret )
		{
			// 幾何形状ファイルの読み込み
			obj = LoadObj( file_name );

			// 読み込んだ幾何形状をスケーリングする（スケーリング後の中心の高さを記録）
			object_y = ScaleObj( obj, 5.0f );

			// 画面の再描画
			glutPostRedisplay();
		}
	}
}


//
//  環境初期化関数
//
void  initEnvironment( void )
{
	// 光源を作成する
	float  light0_position[] = { 10.0, 30.0, 10.0, 1.0 };
	float  light0_diffuse[] = { 0.8, 0.8, 0.8, 1.0 };
	float  light0_specular[] = { 1.0, 1.0, 1.0, 1.0 };
	float  light0_ambient[] = { 0.1, 0.1, 0.1, 1.0 };
	glLightfv( GL_LIGHT0, GL_POSITION, light0_position );
	glLightfv( GL_LIGHT0, GL_DIFFUSE, light0_diffuse );
	glLightfv( GL_LIGHT0, GL_SPECULAR, light0_specular );
	glLightfv( GL_LIGHT0, GL_AMBIENT, light0_ambient );
	glEnable( GL_LIGHT0 );

	// 光源計算を有効にする
	glEnable( GL_LIGHTING );

	// 物体の色情報を有効にする
	glEnable( GL_COLOR_MATERIAL );

	// Ｚテストを有効にする
	glEnable( GL_DEPTH_TEST );

	// 背面除去を有効にする
	glCullFace( GL_BACK );
	glEnable( GL_CULL_FACE );

	// 背景色を設定
	glClearColor( 0.5, 0.5, 0.8, 0.0 );
}


//
//  メイン関数（プログラムはここから開始）
//
int  main( int argc, char ** argv )
{
	// GLUTの初期化
	glutInit( &argc, argv );
	glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGBA );
	glutInitWindowSize( 640, 640 );
	glutInitWindowPosition( 0, 0 );
	glutCreateWindow( "Obj Viewer" );

	// コールバック関数の登録
	glutDisplayFunc( DisplayCallback );
	glutReshapeFunc( ReshapeCallback );
	glutMouseFunc( MouseClickCallback );
	glutMotionFunc( MouseDragCallback );
	glutKeyboardFunc( KeyboardCallback );

	// 環境初期化
	initEnvironment();

	// GLUTのメインループに処理を移す
	glutMainLoop();
	
	// プログラムを終了
	return 0;
}

